import Foundation

// MARK: - TestType

public extension EUDCC.Test {
    
    /// The EUDCC type of test
    struct TestType: Hashable {
        
        // MARK: Properties
        
        /// The string value
        public let value: String
        
        // MARK: Initializer
        
        /// Creates a new instance of `EUDCC.Test.TestType`
        /// - Parameter value: The string value
        public init(value: String) {
            self.value = value
        }
        
    }
    
}

// MARK: - WellKnownValue

public extension EUDCC.Test.TestType {
    
    /// The WellKnownValue
    enum WellKnownValue: String, Codable, Hashable, CaseIterable {
        /// Nucleic acid amplification with probe detection
        case nucleicACIDAmplificationWithProbeDetection = "LP6464-4"
        /// Rapid immunoassay
        case rapidImmunoassay = "LP217198-3"
    }
    
    /// The WellKnownValue if available
    var wellKnownValue: WellKnownValue? {
        .init(rawValue: self.value)
    }
    
}

// MARK: - WellKnownValue+Convenience

public extension EUDCC.Test.TestType.WellKnownValue {
    
    /// PCR represented by `nucleicACIDAmplificationWithProbeDetection` case
    static let pcr: Self = .nucleicACIDAmplificationWithProbeDetection
    
}

// MARK: - Codable

extension EUDCC.Test.TestType: Codable {
    
    /// Creates a new instance by decoding from the given decoder.
    /// - Parameter decoder: The decoder to read data from.
    public init(from decoder: Decoder) throws {
        let container = try decoder.singleValueContainer()
        self.value = try container.decode(String.self)
    }
    
    /// Encodes this value into the given encoder.
    /// - Parameter encoder: The encoder to write data to.
    public func encode(to encoder: Encoder) throws {
        var container = encoder.singleValueContainer()
        try container.encode(self.value)
    }
    
}
